use g::bootstrap::bootstrap;
use g::git::{
    commit, create_new_branch, delete_branch, fetch, git_add, pull, push, revert_commits,
};
use g::gitmoji::{CommitMessage, select_gitmoji};
use g::{NO_COMMIT_REGEX, no_commit_amount, show_rg, show_todos};
use subprocess::Exec;
use yansi::{Color, Paint};

fn main() {
    let args = g::args::get_args();

    match args.subcommand() {
        Some(("bootstrap", bs_args)) => {
            let base = bs_args.get_one::<String>("BASE").unwrap();
            let repo = bs_args.get_one::<String>("NEW").unwrap();
            bootstrap(base, repo);
        }
        Some(("new", new_args)) => {
            let branch: &String = new_args.get_one("BRANCH").unwrap();
            create_new_branch(branch);
        }
        Some(("remove", rm_args)) => {
            let branch: &String = rm_args.get_one("BRANCH").unwrap();
            delete_branch(branch);
        }
        Some(("revert", rev_args)) => {
            let n: &String = rev_args.get_one("N").unwrap();
            let n: u64 = n.parse().unwrap();
            revert_commits(n);
        }
        Some(("branch", branch_args)) => {
            let branch: Option<&String> = branch_args.get_one("BRANCH");

            if let Some(branch) = branch {
                Exec::cmd("git")
                    .arg("checkout")
                    .arg(branch)
                    .popen()
                    .unwrap()
                    .wait()
                    .unwrap();
            } else {
                Exec::cmd("git")
                    .arg("branch")
                    .popen()
                    .unwrap()
                    .wait()
                    .unwrap();
            }
        }
        Some(("push", push_args)) => {
            let force = push_args.get_flag("force");
            push(force);
        }
        Some(("commit", commit_args)) => {
            let all = commit_args.get_flag("all");
            let done = commit_args.get_flag("done");
            let forced = commit_args.get_flag("force");
            let msg: Option<&String> = commit_args.get_one("message");
            let interactive = commit_args.get_flag("interactive");

            if !forced {
                // Impossible commits
                if no_commit_amount() > 0 {
                    println!("{}", "Unable to commit because of:".paint(Color::Red));
                    show_rg(NO_COMMIT_REGEX);
                    std::process::exit(1);
                }
            }

            if interactive || msg.is_none() {
                let gitmoji = select_gitmoji();
                let title = inquire::prompt_text("Commit Title").unwrap();
                let body = inquire::Text::new("Commit Description")
                    .with_page_size(5)
                    .prompt();
                let msg = CommitMessage::new(gitmoji, title, body.ok()).to_msg();
                commit(all, done, forced, &msg);
            } else {
                commit(all, done, forced, msg.unwrap());
            }
        }
        Some(("save", _)) => {
            commit(true, false, true, "WIP");
        }
        Some(("fetch", _)) => {
            fetch();
        }
        Some(("pull", _)) => {
            pull();
        }
        Some(("todo", todo_arg)) => {
            let count = todo_arg.get_flag("count");
            show_todos(count);
        }
        Some(("log", _)) => {
            Exec::cmd("serie").popen().unwrap().wait().unwrap();
        }
        Some(("status", _)) => {
            Exec::cmd("git")
                .arg("status")
                .popen()
                .unwrap()
                .wait()
                .unwrap();
        }
        Some(("add", args)) => {
            let file: &String = args.get_one("FILE").unwrap();
            git_add(file);
        }
        _ => {}
    }
}
